@@ -21,6 +21,7 @@
 #include "core/or/conflux.h"
 #include "core/or/conflux_params.h"
 #include "core/or/conflux_util.h"
+#include "core/or/conflux_pool.h"
 #include "core/or/conflux_st.h"
 #include "core/or/conflux_cell.h"
 #include "lib/time/compat_time.h"
@@ -243,7 +244,9 @@ conflux_decide_circ_minrtt(const conflux_t *cfx)
   tor_assert(CONFLUX_NUM_LEGS(cfx));
 
   CONFLUX_FOR_EACH_LEG_BEGIN(cfx, leg) {
-    if (leg->circ_rtts_usec < min_rtt) {
+
+    /* Ignore circuits with no RTT measurement */
+    if (leg->circ_rtts_usec && leg->circ_rtts_usec < min_rtt) {
       circ = leg->circ;
       min_rtt = leg->circ_rtts_usec;
     }
@@ -278,7 +281,8 @@ conflux_decide_circ_lowrtt(const conflux_t *cfx)
       continue;
     }
 
-    if (leg->circ_rtts_usec < low_rtt) {
+    /* Ignore circuits with no RTT */
+    if (leg->circ_rtts_usec && leg->circ_rtts_usec < low_rtt) {
       low_rtt = leg->circ_rtts_usec;
       circ = leg->circ;
     }
@@ -398,7 +402,8 @@ conflux_decide_circ_cwndrtt(const conflux_t *cfx)
 
   /* Find the leg with the minimum RTT.*/
   CONFLUX_FOR_EACH_LEG_BEGIN(cfx, l) {
-    if (l->circ_rtts_usec < min_rtt) {
+    /* Ignore circuits with invalid RTT */
+    if (l->circ_rtts_usec && l->circ_rtts_usec < min_rtt) {
       min_rtt = l->circ_rtts_usec;
       leg = l;
     }
@@ -419,7 +424,8 @@ conflux_decide_circ_cwndrtt(const conflux_t *cfx)
     /* Pick a 'min_leg' with the lowest RTT that still has
      * room in the congestion window. Note that this works for
      * min_leg itself, up to inflight. */
-    if (cwnd_sendable(l->circ, min_rtt, l->circ_rtts_usec) > 0) {
+    if (l->circ_rtts_usec &&
+        cwnd_sendable(l->circ, min_rtt, l->circ_rtts_usec) > 0) {
       leg = l;
     }
   } CONFLUX_FOR_EACH_LEG_END(l);
@@ -548,11 +554,16 @@ conflux_pick_first_leg(conflux_t *cfx)
     }
   } CONFLUX_FOR_EACH_LEG_END(leg);
 
-  if (BUG(!min_leg)) {
+  if (!min_leg) {
     // Get the 0th leg; if it does not exist, assert
     tor_assert(smartlist_len(cfx->legs) > 0);
     min_leg = smartlist_get(cfx->legs, 0);
     tor_assert(min_leg);
+    if (BUG(min_leg->linked_sent_usec == 0)) {
+      log_warn(LD_BUG, "Conflux has no legs with non-zero RTT. "
+               "Using first leg.");
+      conflux_log_set(cfx, CIRCUIT_IS_ORIGIN(min_leg->circ));
+    }
   }
 
   // TODO-329-TUNING: We may want to initialize this to a cwnd, to
